﻿// DataDot.cs by Charles Petzold, December 2008
using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;

namespace DataLibrary
{
    public class DataDot : FrameworkElement
    {
        public static readonly DependencyProperty CenterXProperty =
            DependencyProperty.Register("CenterX",
                typeof(double),
                typeof(DataDot),
                new FrameworkPropertyMetadata(0.0, 
                        FrameworkPropertyMetadataOptions.AffectsMeasure |
                        FrameworkPropertyMetadataOptions.AffectsRender));

        public static readonly DependencyProperty CenterYProperty =
            DependencyProperty.Register("CenterY",
                typeof(double),
                typeof(DataDot),
                new FrameworkPropertyMetadata(0.0, 
                        FrameworkPropertyMetadataOptions.AffectsMeasure |
                        FrameworkPropertyMetadataOptions.AffectsRender));

        public static readonly DependencyProperty RadiusXProperty =
            DependencyProperty.Register("RadiusX",
                typeof(double),
                typeof(DataDot),
                new FrameworkPropertyMetadata(0.0, 
                        FrameworkPropertyMetadataOptions.AffectsMeasure |
                        FrameworkPropertyMetadataOptions.AffectsRender));

        public static readonly DependencyProperty RadiusYProperty =
            DependencyProperty.Register("RadiusY",
                typeof(double),
                typeof(DataDot),
                new FrameworkPropertyMetadata(0.0, 
                        FrameworkPropertyMetadataOptions.AffectsMeasure |
                        FrameworkPropertyMetadataOptions.AffectsRender));

        public static readonly DependencyProperty BrushesProperty =
            DependencyProperty.Register("Brushes",
                typeof(Brush[]),
                typeof(DataDot),
                new FrameworkPropertyMetadata(null, 
                        FrameworkPropertyMetadataOptions.AffectsRender));

        public static readonly DependencyProperty FillIndexProperty =
            DependencyProperty.Register("FillIndex",
                typeof(int),
                typeof(DataDot),
                new FrameworkPropertyMetadata(0, 
                        FrameworkPropertyMetadataOptions.AffectsRender));

        public double CenterX
        {
            set { SetValue(CenterXProperty, value); }
            get { return (double)GetValue(CenterXProperty); }
        }

        public double CenterY
        {
            set { SetValue(CenterYProperty, value); }
            get { return (double)GetValue(CenterYProperty); }
        }

        public double RadiusX
        {
            set { SetValue(RadiusXProperty, value); }
            get { return (double)GetValue(RadiusXProperty); }
        }

        public double RadiusY
        {
            set { SetValue(RadiusYProperty, value); }
            get { return (double)GetValue(RadiusYProperty); }
        }

        public Brush[] Brushes
        {
            set { SetValue(BrushesProperty, value); }
            get { return (Brush[])GetValue(BrushesProperty); }
        }

        public int FillIndex
        {
            set { SetValue(FillIndexProperty, value); }
            get { return (int)GetValue(FillIndexProperty); }
        }

        protected override Size MeasureOverride(Size availableSize)
        {
            return new Size(CenterX + RadiusX, CenterY + RadiusY);
        }

        protected override void OnRender(DrawingContext dc)
        {
            if (Brushes != null)
                dc.DrawEllipse(Brushes[FillIndex], null, 
                               new Point(CenterX, CenterY), 
                               RadiusX, RadiusY);
        }
    }
}