﻿// DataRandomizer.cs by Charles Petzold, December 2008
using System;
using System.Collections.Generic;
using System.Windows.Threading;

namespace DataLibrary
{
    public class DataRandomizer<T> where T : DataPoint, new()
    {
        IList<T> collection;
        int changePoints;
        Random rand = new Random();

        public DataRandomizer(IList<T> collection, int numPoints, int changePoints)
        {
            this.collection = collection;
            this.changePoints = changePoints;

            for (int i = 0; i < numPoints; i++)
            {
                T dataPoint = new T();
                dataPoint.ID = FabricateIdString();
                dataPoint.Type = rand.Next(6);
                

                switch (dataPoint.Type)
                {
                    case 0:
                        dataPoint.VariableX = rand.NextDouble();
                        dataPoint.VariableY = 0.25 * rand.NextDouble() + 0.17 - 0.125; 
                        break;

                    case 1:
                        dataPoint.VariableX = rand.NextDouble(); 
                        dataPoint.VariableY = 0.25 * rand.NextDouble() + 0.50 - 0.125; 
                        break;

                    case 2:
                        dataPoint.VariableX = rand.NextDouble(); 
                        dataPoint.VariableY = 0.25 * rand.NextDouble() + 0.83 - 0.125; 
                        break;

                    case 3:
                        dataPoint.VariableX = rand.NextDouble();
                        dataPoint.VariableY = 0.75 * dataPoint.VariableX + 0.25 * rand.NextDouble();
                        break;

                    case 4:
                        dataPoint.VariableX = rand.NextDouble();
                        dataPoint.VariableY = 0.75 * (1 - dataPoint.VariableX) + 0.25 * rand.NextDouble();
                        break;

                    case 5:
                        dataPoint.VariableX = 0.5 * rand.NextDouble() + 0.25;
                        dataPoint.VariableY = rand.NextDouble();
                        break;
                }

                collection.Add(dataPoint);
            }

            DispatcherTimer tmr = new DispatcherTimer();
            tmr.Interval = TimeSpan.FromMilliseconds(100);
            tmr.Tick += OnTimerTick;
            tmr.Start();
        }

        string FabricateIdString()
        {
            string str = "";
            int num = rand.Next(3, 11);
            char[] vowels = { 'A', 'E', 'I', 'O', 'U' };
            char[] cons = { 'B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'L',
                            'M', 'N', 'P', 'R', 'S', 'T', 'V', 'W', 'Z' };

            for (int i = 0; i < num; i++)
            {
                char ch = (i % 2 == 0) ? cons[rand.Next(cons.Length)] : 
                                         vowels[rand.Next(vowels.Length)];
                str += (i == 0) ? ch : Char.ToLower(ch);
            }
            return str;
        }

        void OnTimerTick(object sender, EventArgs args)
        {
            for (int i = 0; i < changePoints; i++)
            {
                int index = rand.Next(collection.Count);
                T dataPoint = collection[index];
                double change = 0.1 * (rand.NextDouble() - 0.5);

                if (rand.Next(2) == 0)
                    dataPoint.VariableX =
                        Math.Max(0, Math.Min(1, dataPoint.VariableX + change));
                else
                    dataPoint.VariableY = 
                        Math.Max(0, Math.Min(1, dataPoint.VariableY + change));
            }
        }
    }
}